"""
Библиотека для сервиса
https://vkmix.com/settings/api
"""

import requests, urllib.parse, json


class VkMixException(Exception):
	"""[summary]

	Args:
		Exception ([type]): [description]
	"""
	pass


class VkMixApiError(VkMixException):
	"""[summary]

	Args:
		VkMixException ([type]): [description]
	"""
	pass


class VkMix:
	"""API для ботов API VKMix
Мы предоставляем открытый для всех разработчиков доступ к созданию заданий в нашей системе.

Взаимодействие с API
Всем методам необходимо передавать токен авторизации параметром api_token}.
Успешный результат в виде
{response: результат работы метода}
Ошибки:
{error: тип ошибки}
Список доступных методов"""

	s = requests.session()
	ua = r"Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:79.0) Gecko/20100101 Firefox/79.0"
	s.headers = {"User-Agent": ua, "X-Requested-With": "XMLHttpRequest"}
	api_token = ""
	url = "https://vkmix.com/api/2/"

	def __init__(self, api_token):
		"Авторизация"
		self.api_token = api_token

	def _request(self, uri, method="get", data={}, headers={}, **kw):
		method = method.lower()
		if method not in ["get", "post"]:
			raise ValueError("method GET or POST")

		kw.update({"api_token": self.api_token})
		data.update(kw)

		if method == "get":
			hndlr = self.s.get
			uri = uri + "?" + urllib.parse.urlencode(data)
			data = {}

		if method == "post":
			hndlr = self.s.post

		try:
			resp = hndlr(
				self.url + uri, data=data, headers=headers, allow_redirects=False
			).content.decode("utf8")
		except:
			raise

		if resp == "":
			raise VkMixApiError("empty response")

		try:
			j = json.loads(resp)
		except json.JSONDecodeError:
			raise VkMixApiError("not json" + resp)

		if "error" in j and not "response" in j:
			raise VkMixApiError(j["error"])

		return j["response"]

	def getServices(self):
		"""getServices

Получение списка сервисов
Результат
Метод возвращает список сервисов для создания заданий"""
		
		return self._request("getServices")
	
	def createTask(self, **kw):
		"""Добавление нового задания

Параметры
network
Социальная сеть задания. Укажите одно из значений:
vk - ВКонтакте
instagram - Инстаграм
youtube - Ютуб
telegram - Телеграм
ok - Одноклассники
twitter - Твиттер

section
Тип задания. Для каждой социальной сети доступны свои типы:
vk: likes, reposts, comments, friends, groups, polls
instagram: likes, subscribers, comments, comments_likes
youtube: likes, friends, dislikes, comments
twitter: retweets, followers, favorites
ok: likes, friends, groups
telegram: subscribers
Для Instagram дополнительно доступны: likes_q4, subscribers_q4, likes_q5, subscribers_q5, likes_q7, subscribers_q7. 

link
Ссылка на объект задания.

count
Количество необходимых выполнений.

amount
Вознаграждение пользователю за выполнение задания.

comments (опц. для section = comments)
Массив вариантов комментариев
	# todo: массив comments может передаваться не корректно

poll (опц. для section = polls)
Номер варианта за который необходимо проголосовать

hourly_limit
Лимит выполнений в час

Результат
Метод возвращает ID созданного задания."""

		return self._request("createTask", method="post", data=kw)

	def getTasks(self, ids="", count=100, offset=0):
		"""Получение списка заданий

Параметры
ids
Id заданий. Если не передан - вернёт все задания

count
Количество заданий, которые необходимо вернуть. Не более 100

offset
Смещение необходимое для выборки определенного подмножества

Результат
Метод возвращает список заданий."""

		return self._request("getTasks", ids=ids, count=count, offset=offset)

	def getBalance(self):
		"""Получение текущего баланса аккаунта

Результат
Метод возвращает баланс аккаунта."""

		return self._request("getBalance")



import responses
import requests
import urllib.parse
import json
import unittest
from vkmix import VkMix

class TestVkMixCreateTask(unittest.TestCase):
    success_data = json.loads(r"""
{"response":{"success": 1, "points": 50, "id": 30592809}}
""".strip())
    
    def response_callback(self, resp):
        resp.callback_processed = True
        return resp
        args = {}
        try:
            args = urllib.parse.parse_qs(urllib.parse.urlparse(resp.url)[4])
        except AttributeError: pass
        except KeyError: pass
        try: args.update(resp.data)
        except TypeError: pass
        
        self.assertIn("api_token", args)
        self.assertEqual(args["api_token"][0], "mykey")
        self.assertEqual(args["network"][0], "vk")
        self.assertEqual(args["section"][0], "likes")
        self.assertEqual(args["link"][0], "https://vk.com/wall-139740824_2687166")
        self.assertEqual(int(args["count"][0]), 10)
        self.assertEqual(int(args["hourly_limit"][0]), 5)
        self.assertEqual(int(args["amount"][0]), 5)
        return resp

    def test_createTask(self):
        with responses.RequestsMock(response_callback=self.response_callback) as m:
            m.add(responses.POST, "https://vkmix.com/api/2/createTask", json=self.success_data)
            vkm = VkMix(api_token="mykey")
            data = vkm.createTask(
                network = "vk",
                section = "likes",
                link = "https://vk.com/wall-139740824_2687166",
                count = 10,
                hourly_limit = 5,
                amount = 5
            )
            self.assertEqual(m.assert_call_count("https://vkmix.com/api/2/createTask", 1), True)
            self.assertIn("id", data)
            self.assertEqual(data["success"], 1)
            self.assertEqual(data["points"], 50)

if __name__ == "__main__":
    unittest.main()
    


import responses
import requests
import urllib.parse
import json
import unittest
from vkmix import VkMix

class TestVkMixGetTasks(unittest.TestCase):
    success_data = json.loads(r"""
{"response":{"count":3,"items":[{"id":30587450,"done_count":6,"ordered_count":5,"amount":5,"title":"NameA LastNameA","status":"success","source":"api","network":"vk","section":"friends","url":"https:\/\/vk.com\/id1"},{"id":30307135,"done_count":5,"ordered_count":5,"amount":5,"title":"NameA LastNameA","status":"success","source":"api","network":"vk","section":"friends","url":"https:\/\/vk.com\/id1"},{"id":30307090,"done_count":5,"ordered_count":5,"amount":5,"title":"NameB LastNameB","status":"success","source":"api","network":"vk","section":"friends","url":"https:\/\/vk.com\/id2"}]}}
""".strip())
    
    def response_callback(self, resp):
        resp.callback_processed = True
        args = {}
        try:
            args = urllib.parse.parse_qs(urllib.parse.urlparse(resp.url)[4])
        except AttributeError: pass
        except KeyError: pass
        self.assertIn("api_token", args)
        self.assertEqual(args["api_token"][0], "mykey")
        return resp

    def test_getTasks(self):
        with responses.RequestsMock(response_callback=self.response_callback) as m:
            m.add(responses.GET, "https://vkmix.com/api/2/getTasks", json=self.success_data)
            vkm = VkMix(api_token="mykey")
            data = vkm.getTasks()
            # self.assertEqual(m.assert_call_count("https://vkmix.com/api/2/getTasks", 1), True) # no support query string?
            self.assertIn("count", data)
            self.assertIn("items", data)

if __name__ == "__main__":
    unittest.main()
    


import responses
import requests
import urllib.parse
import unittest
from vkmix import VkMix

class TestVkMixGetBalance(unittest.TestCase):
    def response_callback(self, resp):
        resp.callback_processed = True
        args = {}
        try:
            args = urllib.parse.parse_qs(urllib.parse.urlparse(resp.url)[4])
        except AttributeError: pass
        except KeyError: pass
        self.assertIn("api_token", args)
        self.assertEqual(args["api_token"][0], "mykey")
        return resp

    def test_getBalance(self):
        with responses.RequestsMock(response_callback=self.response_callback) as m:
            m.add(responses.GET, "https://vkmix.com/api/2/getBalance", json={"response":24.64})
            vkm = VkMix(api_token="mykey")
            data = vkm.getBalance()
            # self.assertEqual(m.assert_call_count("https://vkmix.com/api/2/getBalance", 1), True) # no support query string?
            self.assertEqual(data, 24.64)

if __name__ == "__main__":
    unittest.main()
    


from setuptools import setup, find_packages
from os.path import join, dirname

setup(
    name="vkmix",
    version="1.1",
    author="alekssamos",
    author_email="aleks-samos@yandex.ru",
    url="https://github.com/alekssamos/vkmix/",
    packages=find_packages(),
    include_package_data=True,
    long_description_content_type="text/markdown",
    long_description=open(join(dirname(__file__), "README.MD"), encoding="UTF8").read(),
)
